/**
  ******************************************************************************
  * @file           : main.c
  * @author         : sinomcu.com-AE
  * @brief          : Main program body
  *    
  *          This file provides example code for UART IDLEIE and UART DMA transfer,printf by uart,
  * LED1 blink by sysTick soft timer,if data transfer ok LED blink period 1S, DMA ERROR LED
  * blink period 0.25 S.
  * For details, see "readme.txt"   
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */  
  
#include "system_define.h"

#define SYSCLK_USE_PLL_48M        1         // 1:sysclk PLL48M,0 or not define:HSI 8M
#define BAUD_RATE                 115200    // unit bps       
#define LED_BLINK_HALF_PER        500       // unit ms,LED blink half cycle

uint8_t uart_buff[UART_BUFF_LEN]={0};
FlagStatus DMA_ReceptionComplete_Flag = RESET;
FlagStatus DMA_TransferError_Flag = RESET;
FlagStatus UART_IDIE_Flag = RESET; 
uint16_t  revc_data_num=0;

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void) {
  uint32_t LED_blink_Time = LED_BLINK_HALF_PER;
  uint16_t uart_recv_num = 0;
  
#if(defined(SYSCLK_USE_PLL_48M) && SYSCLK_USE_PLL_48M) 
  sysclk_init();                            // sysclk 48M,HSI 8M /2*12;if not call this function,default HSI 8M
#endif
  SystemCoreClockUpdate();
  
  sys_tick_init();
  gpio_init();
  dma_uart_rx_init();
 
  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_HSI);       // uart clk HSI 8M     
  usart1_uart_init(BAUD_RATE);                                       // AN2102,the other side 2 STOP

  LED1_ON(); 
  
  printf("\r\n\r\n*****UART Receive DMA IDLE Example*****");
  printf("\r\n--Inf:System Core Clock %d Hz.\n",SystemCoreClock);
  
  Start_User_MS_SF_Timer(LED_blink_Time);
  while(1) {
    if(DMA_ReceptionComplete_Flag == SET || UART_IDIE_Flag == SET) { // DMA transfer Complete
      if(DMA_ReceptionComplete_Flag == SET) {
        uart_recv_num = UART_BUFF_LEN;
      }
      if(UART_IDIE_Flag == SET) {
        uart_recv_num = revc_data_num;
      }
      DMA_ReceptionComplete_Flag = RESET;
      UART_IDIE_Flag = RESET;
      LED_blink_Time = LED_BLINK_HALF_PER;
      Send_buff_Uart(uart_buff,uart_recv_num);
    }
    else if(DMA_TransferError_Flag == SET) {     // DMA transfer
      DMA_TransferError_Flag = RESET;
      LED_blink_Time =  LED_blink_Time >>2;                          // DMA transfer ERROR LED blink fast
      printf("\r\n**Error:DMA Transfer Fail,DAM Channel 3 is disabled.");
    }
    
    if(Get_State_User_MS_SF_Timer() == SF_TIMER_OV) {
      Start_User_MS_SF_Timer(LED_blink_Time);                        // clear soft timer ov
      LED1_TOGGLE();                        
    }
  }
}

/**
  * @brief  SystemClock_Config HSI/2 to PLL*12 sysclk 48M
  * @param  None
  * @retval None
  */
void sysclk_init(void) {
  MS32_RCC_OscInitTypeDef osc_cfg;
  MS32_RCC_PllInitTypeDef pll_cfg;
  MS32_RCC_ClkInitTypeDef clk_cfg;

  // get and clear reset flag 
  MS32_RCC_GetResetFlag(ENABLE);
  
  // osc config 
  osc_cfg.HSIState = MS32_RCC_HSI_ON;
  osc_cfg.HSEState = MS32_RCC_HSE_OFF;
  osc_cfg.LSIState = MS32_RCC_LSI_OFF;
  osc_cfg.LSEState = MS32_RCC_LSE_OFF;
  MS32_RCC_OscConfig(&osc_cfg);
  
  // pll config 
  pll_cfg.PLLState  = MS32_RCC_PLL_ON;
  pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSI_DIV_2;  // HSI/2 4M
  pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL12;
  pll_cfg.PREDIV    = MS32_RCC_PLL_PREDIV_DIV1;
  MS32_RCC_PllConfig(&pll_cfg);
  
  // sysclk config 
  clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_PLL;
  clk_cfg.AHBCLKDivider  = MS32_RCC_SYSCLK_DIV1;
  clk_cfg.APB1CLKDivider = MS32_RCC_HCLK_DIV1;
  MS32_RCC_ClockConfig(&clk_cfg);
}
/******************************** END OF FILE *********************************/
